var fs = require('fs')
var path = require('path')
var rpc = require('rpc-stream')
var merge = require('merge')
var storage = require('../storage')
const bole = require('bole')
const logger = bole(process.title)('worker')

var argv = require('yargs')
  .usage('Usage: $0')
  .help('help')
  .describe('storageUrl', 'the storage server address, e.g. https://localhost:4000')
  .describe('storageAPIVersion', 'the storage server API endpoint, e.g. /api/v1')
  .alias('help', 'h')
  .argv

/*

   WebInspector code (minus shims) are from:

   git clone --depth=1 https://chromium.googlesource.com/chromium/src.git ~/chromium/src/
   There are (currently) 6 files that are taken verbatim, and only get a `require`
     statement inserted to keep them modular. They are:

   * HeapSnapshot.js
   * HeapSnapshotLoader.js
   * HeapSnapshotCommon.js
   * JSHeapSnapshot.js
   * TextUtils.js
   * utilities.js

   The current version is git sha: d76422e697ad4de2edac5b23f12da1ee5e9dcd56
     (April 13th)
*/

var WebInspector = require('./webinspector');
require('./HeapSnapshot');
require('./HeapSnapshotLoader');
require('./HeapSnapshotCommon');
require('./JSHeapSnapshot');
require('./TextUtils');

var snapshot = null

// TODO: there is a potentially huge perf boost here if we can figure out
//       how to disect this snapshot thing into a queryable structure that
//       lives on disk


// TODO: add non-constructor based filtering support
function serializeChildren(parentId, sort, pager, retainedMode) {
  if (0 === parentId) {
    var rootStuff = snapshot.aggregatesWithFilter({});

    return {
      items: Object.keys(rootStuff).map(function (key, i) {
        var root = rootStuff[key];
        var name = root.name || key;
        var output = {
          retainedSize: root.maxRet,
          selfSize: root.self,
          count: root.count || 0,
          nodeName: root.name + (root.nodeIndex ? '@' + root.nodeIndex : ''),
          empty: root.count === 0,
          type: root.type,
          name: name,
          nodeIndex: name
        };

        return output;
      }).sort(function(a, b) {
          return b.retainedSize - a.retainedSize
      })
    };

  } else if (retainedMode) {
    var nodesProvider = snapshot.createRetainingEdgesProvider(parentId);

    nodesProvider.sortAndRewind({
      "fieldName1":"distance",
      "ascending1":true,
      "fieldName2":"_name",
      "ascending2":true
    })

    var result = nodesProvider.serializeItemsRange(0, 1000);

    result.items = result.items.map(function(item) {
      if (item.node) {
        var name = item.name;
        var nodeIndex = item.edgeIndex
        item = item.node;

        item.name = name;
        item.nodeIndex = nodeIndex;
      }

      if (item.type === 'closure' && !item.name) {
        item.name = 'function()'
      }

      if (item.type === 'array') {
        item.name += ' []'
      }

      // TODO: cleanup memory or cache properly!
      var empty = item.type === 'string' || snapshot.createEdgesProvider(item.nodeIndex).isEmpty();

      return {
        id: item.id,
        selfSize: item.selfSize,
        retainedSize: item.retainedSize,
        count: 0, // count is only computed on root nodes
        nodeName: item.name + (item.nodeIndex ? '@' + item.nodeIndex : ''),
        type: item.type,
        name: item.name,
        nodeIndex: item.nodeIndex
      }
    });
    return result;

  } else {
    var nodesProvider;

    if (parentId.length > 0) {
      nodesProvider = snapshot.createNodesProviderForClass(parentId,{});
    } else {
      nodesProvider = snapshot.createEdgesProvider(parentId)
    }

    // TODO: handle leaves via `!edgeName` as fieldName2

    if (nodesProvider.isEmpty()) {
      sortOptions.fieldName2 = '!edgeName';
      // defaultSearch.ascending2 = false;
    }

    nodesProvider.sortAndRewind(sort)

    // TODO: implement pagination
    var result = nodesProvider.serializeItemsRange(pager.start, pager.start + pager.size);

    result.items = result.items.map(function(item) {
      if (item.node) {
        var name = item.name;
        var nodeIndex = item.edgeIndex
        item = item.node;

        item.name = name;
        item.nodeIndex = nodeIndex;
      }

      if (item.type === 'closure' && !item.name) {
        item.name = 'function()'
      }

      if (item.type === 'array') {
        item.name += ' []'
      }

      // TODO: cleanup memory or cache properly!
      var empty = item.type === 'string' || snapshot.createEdgesProvider(item.nodeIndex).isEmpty();

      return {
        id: item.id,
        selfSize: item.selfSize,
        retainedSize: item.retainedSize,
        count: 0, // count is only computed on root nodes
        nodeName: item.name + (item.nodeIndex ? '@' + item.nodeIndex : ''),
        type: item.type,
        name: item.name,
        nodeIndex: item.nodeIndex
      }
    });
    return result;
  }
}
process.stdin.pipe(rpc({
  loadFile: function(asset, cb) {
    var snapshotLoader = new WebInspector.HeapSnapshotLoader()

    snapshotLoader._progress = {
      updateStatus: function(status) {
        currentStatus = status;
        if (status === "Finished processing.") {
          process.stdout.write('!progress!' + JSON.stringify({
            type: 'done'
          }) + '\n');
        } else {
          process.stdout.write('!progress!' + JSON.stringify({
            type: 'status',
            status: status,
            value: 0
          }) + '\n');
        }
      },
      updateProgress: function(title, value, total) {
        process.stdout.write('!progress!' + JSON.stringify({
          type: 'progress',
          status: currentStatus,
          value: value / total
        }) + '\n');
      },
      notifyDone: function () {
        // This event has been removed, now sends a "Finished processing."
        // updateStatus event.
        process.stdout.write('!progress!' + JSON.stringify({
          type: 'done'
        }) + '\n');
      },
      _sendUpdateEvent: function(text) {
        logger.error('unexpected _sendUpdateEvent: %s', text)
      }
    }

    const readStream = storage.createAssetStream(argv, {
      asset: asset
    })

    readStream.on('response', res => {
      if (res.statusCode !== 200) {
        var err = new Error('Error fetching snapshot')
        err.code = res.statusCode
        cb(err)

        // Since this can also produce errors, pass our error first
        readStream.destroy()
      }
    })

    readStream.on('data', function(chunk) {
      try {
        snapshotLoader.write(chunk.toString());
      } catch (e) {
        this.emit('error', e)
        this.destroy()
      }
    }).on('end', function() {
      try {
        snapshotLoader.close();
        snapshot = snapshotLoader.buildSnapshot(true)
        cb()
      } catch (e) {
        cb(e)
      }
    }).on('error', function(err) {
      cb(err)
    })
  },


  getNodeData : function(req, cb) {
    if (!snapshot) {
      return cb({
        message: 'snapshot could not be loaded'
      })
    }

    var id = req.id
    var sort = req.sort || [['retainedSize', true]]

    var sortOptions = {
      "fieldName1":sort[0][0],
      "ascending1":sort[0][1],
      "fieldName2":"id",
      "ascending2":true
    };

    if (sort[0][0] === 'count') {
      sortOptions = {
        "fieldName2":"id",
        "ascending2":false,
        "fieldName1":'retainedSize',
        "ascending1":false,
      };
    }

    var pager = {
      start: req.start || 0,
      size: req.size || 100
    }

    try {
      var r = serializeChildren(id, sortOptions, pager, !!req.retainedMode);
      cb(null, r)
    } catch (e) {
      logger.error(e);
      cb(e);
    }
  }
})).pipe(process.stdout);
