'use strict'

const EventEmitter = require('events')

const util = require('./util')

const Logger = util.getLogger(__dirname, __filename)

exports.create = create

function create (context) {
  return new AssetManager(context)
}

class AssetManager extends EventEmitter {
  constructor (context) {
    super()

    this._context = context
    this._assets = new Map()
  }

  byId (bulkId) {
    return this._assets.get(bulkId)
  }

  add (asset) {
    Logger.debug('Adding asset', `${asset}`)
    this._assets.set(asset.bulkId, asset)
    this.emit(`add:${asset.bulkId}`, asset)
    this.emit('add', asset)
  }

  // This will happen when the Asset has finished writing the file to disk.
  remove (asset) {
    Logger.debug('Removing asset', `${asset}`)
    this._assets.delete(asset.bulkId)
    this.emit(`remove:${asset.bulkId}`, asset)
    this.emit('remove', asset)
  }
}
