'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const qData = req.nsStorage.qData
  const licenseKey = qData.license

  if (licenseKey.length === 0) {
    res.status(404)
    res.send({ error: { message: 'Invalid key.' } })
    return
  }

  if (context.provider.isOptedOut()) {
    res.status(400)
    res.send({
      error: {
        message: 'Bad request. Can not check license in offline mode.'
      }
    })
    return
  }

  context.provider.validateKey(licenseKey, (err, license) => {
    if (err) {
      res.status(503)
      res.send({ error: { message: err.message } })
      Logger.error(err, `Error checking license ${licenseKey}.`)
      return
    }

    if (!license) {
      res.status(404)
      res.send({
        time: Date.now(),
        valid: false
      })
      return
    }

    res.status(200)
    res.send({
      time: Date.now(),
      valid: license.valid
    })
  })
}
