'use strict'

exports.handleRequest = handleRequest

const util = require('../util')
const simpleCommand = require('./simple-command')

const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsStorage.context
  const eventStream = req.nsStorage.eventStream
  const qData = req.nsStorage.qData

  simpleCommand.run(context, eventStream, qData, {
    command: 'info',
    transformer: getTransformer(context)
  })
}

class Transformer {
  constructor (context) {
    this._context = context
  }

  transform (body, cb) {
    const agent = this._context.agentManager.byId(body.id)

    if (!agent) {
      return cb(null, body)
    }

    body.app = agent.tags.app
    body.hostname = agent.tags.hostname
    if (typeof body.versions === 'string') {
      try {
        body.versions = JSON.parse(body.versions)
      } catch (err) {
        Logger.error('Can not parse info.versions', err)
      }
    }
    cb(null, body)
  }
}

function getTransformer (context) {
  const transformer = new Transformer(context)
  return function doTransform (body, cb) {
    transformer.transform(body, cb)
  }
}
