'use strict'

exports.handleRequest = handleRequest

const lodash = require('lodash')

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  // check if body parser set an error
  if (req.nsStorage.bodyErr) {
    return errorReturn(res, 'invalid JSON request body')
  }

  let newSettings = req.body

  const settingsManager = req.nsStorage.context.settingsManager
  const oldSettings = settingsManager.getSettings()

  // update newSettings with remaining settings from oldSettings
  lodash.defaults(newSettings, oldSettings)

  // delete any null keys
  newSettings = lodash.omitBy(newSettings, (val) => val == null)

  // update the settings with settingsManager
  settingsManager.setSettings(newSettings, (err) => {
    if (err) {
      Logger.error(err, 'error updating setttings')
      return errorReturn(res, 'error updating settings')
    }

    if (newSettings.security != null) {
      // This will recalculate the vulnerabilities and send any emails
      // it means new emails will get a report immediately...
      // but so will emails that were already there.
      req.nsStorage.context.vulnManager.rebuildVulnsCache()
    }

    res.status(200)
    res.send({time: Date.now()})
  })
}

// Standard processing for an error return.
function errorReturn (res, message) {
  res.status(400)
  res.send({error: {
    message: message
  }})
}
