'use strict'

const crypto = require('crypto')
const lodash = require('lodash')

exports.hash = function (packages) {
  packages = lodash.cloneDeep(packages)
  const hash = crypto.createHash('sha1')

  // Arrays of different orders should be considered similar enough.
  // These arrays consist of elements that are either a string or an object.
  // We can't sort on the path bc we don't want it to affect the shasum.
  // Sort on name@version or shasum.
  packages = lodash.sortBy(packages, item => `${item.name}@${item.version}`)

  packages.forEach(pkg => {
    delete pkg.path
  })

  const string = stringify(packages)
  hash.update(string)
  return hash.digest('hex')
}

function stringify (obj) {
  let string = ''
  Object.keys(obj).sort().forEach(key => {
    const value = obj[key]
    if (value === null) {
      string += 'null'
    } else if (lodash.isArray(value)) {
      string += value.join(',')
    } else if (typeof value === 'object') {
      string += stringify(value)
    }
    string += value.toString()
  })
  return string
}
